/****************************************************************************
** Meta object code from reading C++ file 'XYChart.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kquickcharts-6.28.0/src/XYChart.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'XYChart.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7XYChartE_t {};
} // unnamed namespace

template <> constexpr inline auto XYChart::qt_create_metaobjectdata<qt_meta_tag_ZN7XYChartE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "XYChart",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Base Class",
        "directionChanged",
        "",
        "stackedChanged",
        "computedRangeChanged",
        "xRange",
        "RangeGroup*",
        "yRange",
        "direction",
        "Direction",
        "stacked",
        "ZeroAtStart",
        "ZeroAtEnd"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'directionChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stackedChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'computedRangeChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'xRange'
        QtMocHelpers::PropertyData<RangeGroup*>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'yRange'
        QtMocHelpers::PropertyData<RangeGroup*>(13, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'direction'
        QtMocHelpers::PropertyData<enum Direction>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'stacked'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Direction'
        QtMocHelpers::EnumData<enum Direction>(15, 15, QMC::EnumIsScoped).add({
            {   17, Direction::ZeroAtStart },
            {   18, Direction::ZeroAtEnd },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<XYChart, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject XYChart::staticMetaObject = { {
    QMetaObject::SuperData::link<Chart::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7XYChartE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7XYChartE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7XYChartE_t>.metaTypes,
    nullptr
} };

void XYChart::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<XYChart *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->directionChanged(); break;
        case 1: _t->stackedChanged(); break;
        case 2: _t->computedRangeChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (XYChart::*)()>(_a, &XYChart::directionChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (XYChart::*)()>(_a, &XYChart::stackedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (XYChart::*)()>(_a, &XYChart::computedRangeChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<RangeGroup**>(_v) = _t->xRange(); break;
        case 1: *reinterpret_cast<RangeGroup**>(_v) = _t->yRange(); break;
        case 2: *reinterpret_cast<enum Direction*>(_v) = _t->direction(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->stacked(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setDirection(*reinterpret_cast<enum Direction*>(_v)); break;
        case 3: _t->setStacked(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *XYChart::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XYChart::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7XYChartE_t>.strings))
        return static_cast<void*>(this);
    return Chart::qt_metacast(_clname);
}

int XYChart::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Chart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void XYChart::directionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void XYChart::stackedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void XYChart::computedRangeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
