/****************************************************************************
** Meta object code from reading C++ file 'Chart.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kquickcharts-6.28.0/src/Chart.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Chart.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5ChartE_t {};
} // unnamed namespace

template <> constexpr inline auto Chart::qt_create_metaobjectdata<qt_meta_tag_ZN5ChartE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Chart",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Base class",
        "nameSourceChanged",
        "",
        "shortNameSourceChanged",
        "colorSourceChanged",
        "valueSourcesChanged",
        "indexingModeChanged",
        "highlightChanged",
        "dataChanged",
        "insertValueSource",
        "index",
        "ChartDataSource*",
        "source",
        "removeValueSource",
        "nameSource",
        "shortNameSource",
        "colorSource",
        "valueSources",
        "QQmlListProperty<ChartDataSource>",
        "indexingMode",
        "IndexingMode",
        "highlight",
        "IndexSourceValues",
        "IndexEachSource",
        "IndexAllValues"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'nameSourceChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shortNameSourceChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorSourceChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'valueSourcesChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'indexingModeChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'highlightChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dataChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'insertValueSource'
        QtMocHelpers::MethodData<void(int, ChartDataSource *)>(15, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { 0x80000000 | 17, 18 },
        }}),
        // Method 'removeValueSource'
        QtMocHelpers::MethodData<void(int)>(19, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Method 'removeValueSource'
        QtMocHelpers::MethodData<void(QObject *)>(19, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'nameSource'
        QtMocHelpers::PropertyData<ChartDataSource*>(20, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'shortNameSource'
        QtMocHelpers::PropertyData<ChartDataSource*>(21, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'colorSource'
        QtMocHelpers::PropertyData<ChartDataSource*>(22, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'valueSources'
        QtMocHelpers::PropertyData<QQmlListProperty<ChartDataSource>>(23, 0x80000000 | 24, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'indexingMode'
        QtMocHelpers::PropertyData<enum IndexingMode>(25, 0x80000000 | 26, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'highlight'
        QtMocHelpers::PropertyData<int>(27, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 5),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'IndexingMode'
        QtMocHelpers::EnumData<enum IndexingMode>(26, 26, QMC::EnumFlags{}).add({
            {   28, IndexingMode::IndexSourceValues },
            {   29, IndexingMode::IndexEachSource },
            {   30, IndexingMode::IndexAllValues },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<Chart, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Chart::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5ChartE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5ChartE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5ChartE_t>.metaTypes,
    nullptr
} };

void Chart::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Chart *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->nameSourceChanged(); break;
        case 1: _t->shortNameSourceChanged(); break;
        case 2: _t->colorSourceChanged(); break;
        case 3: _t->valueSourcesChanged(); break;
        case 4: _t->indexingModeChanged(); break;
        case 5: _t->highlightChanged(); break;
        case 6: _t->dataChanged(); break;
        case 7: _t->insertValueSource((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ChartDataSource*>>(_a[2]))); break;
        case 8: _t->removeValueSource((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->removeValueSource((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ChartDataSource* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Chart::*)()>(_a, &Chart::nameSourceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Chart::*)()>(_a, &Chart::shortNameSourceChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Chart::*)()>(_a, &Chart::colorSourceChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Chart::*)()>(_a, &Chart::valueSourcesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Chart::*)()>(_a, &Chart::indexingModeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Chart::*)()>(_a, &Chart::highlightChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Chart::*)()>(_a, &Chart::dataChanged, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ChartDataSource* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<ChartDataSource**>(_v) = _t->nameSource(); break;
        case 1: *reinterpret_cast<ChartDataSource**>(_v) = _t->shortNameSource(); break;
        case 2: *reinterpret_cast<ChartDataSource**>(_v) = _t->colorSource(); break;
        case 3: *reinterpret_cast<QQmlListProperty<ChartDataSource>*>(_v) = _t->valueSourcesProperty(); break;
        case 4: *reinterpret_cast<enum IndexingMode*>(_v) = _t->indexingMode(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->highlight(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setNameSource(*reinterpret_cast<ChartDataSource**>(_v)); break;
        case 1: _t->setShortNameSource(*reinterpret_cast<ChartDataSource**>(_v)); break;
        case 2: _t->setColorSource(*reinterpret_cast<ChartDataSource**>(_v)); break;
        case 4: _t->setIndexingMode(*reinterpret_cast<enum IndexingMode*>(_v)); break;
        case 5: _t->setHighlight(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 5: _t->resetHighlight(); break;
        default: break;
        }
    }
}

const QMetaObject *Chart::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Chart::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5ChartE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int Chart::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Chart::nameSourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Chart::shortNameSourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Chart::colorSourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Chart::valueSourcesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Chart::indexingModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Chart::highlightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Chart::dataChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
