/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<AxisLabels.h>)
#  include <AxisLabels.h>
#endif
#if __has_include(<GridLines.h>)
#  include <GridLines.h>
#endif
#if __has_include(<LegendLayout.h>)
#  include <LegendLayout.h>
#endif
#if __has_include(<LegendModel.h>)
#  include <LegendModel.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_quickcharts_controls()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<AxisLabels>("org.kde.quickcharts.controls", 1);
    qmlRegisterAnonymousType<QQuickItem, 254>("org.kde.quickcharts.controls", 1);
    qmlRegisterEnum<AxisLabels::Direction>("AxisLabels::Direction");
    qmlRegisterTypesAndRevisions<AxisLabelsAttached>("org.kde.quickcharts.controls", 1);
    qmlRegisterTypesAndRevisions<GridLines>("org.kde.quickcharts.controls", 1);
    qmlRegisterEnum<GridLines::Direction>("GridLines::Direction");
    qmlRegisterTypesAndRevisions<LegendLayout>("org.kde.quickcharts.controls", 1);
    qmlRegisterTypesAndRevisions<LegendLayoutAttached>("org.kde.quickcharts.controls", 1);
    qmlRegisterTypesAndRevisions<LegendModel>("org.kde.quickcharts.controls", 1);
    qmlRegisterEnum<LegendModel::SourceIndex>("LegendModel::SourceIndex");
    qmlRegisterTypesAndRevisions<LinePropertiesGroup>("org.kde.quickcharts.controls", 1);
    QMetaType::fromType<QAbstractItemModel *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    QMetaType::fromType<QAbstractListModel *>().id();
    QT_WARNING_POP
    qmlRegisterModule("org.kde.quickcharts.controls", 1, 0);
}

static const QQmlModuleRegistration orgkdequickchartscontrolsRegistration("org.kde.quickcharts.controls", qml_register_types_org_kde_quickcharts_controls);
