/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<ArraySource.h>)
#  include <ArraySource.h>
#endif
#if __has_include(<BarChart.h>)
#  include <BarChart.h>
#endif
#if __has_include(<Chart.h>)
#  include <Chart.h>
#endif
#if __has_include(<ChartAxisSource.h>)
#  include <ChartAxisSource.h>
#endif
#if __has_include(<ChartDataSource.h>)
#  include <ChartDataSource.h>
#endif
#if __has_include(<ColorGradientSource.h>)
#  include <ColorGradientSource.h>
#endif
#if __has_include(<HistoryProxySource.h>)
#  include <HistoryProxySource.h>
#endif
#if __has_include(<LineChart.h>)
#  include <LineChart.h>
#endif
#if __has_include(<MapProxySource.h>)
#  include <MapProxySource.h>
#endif
#if __has_include(<ModelSource.h>)
#  include <ModelSource.h>
#endif
#if __has_include(<PieChart.h>)
#  include <PieChart.h>
#endif
#if __has_include(<RangeGroup.h>)
#  include <RangeGroup.h>
#endif
#if __has_include(<SingleValueSource.h>)
#  include <SingleValueSource.h>
#endif
#if __has_include(<XYChart.h>)
#  include <XYChart.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_quickcharts()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<ArraySource>("org.kde.quickcharts", 1);
    qmlRegisterTypesAndRevisions<BarChart>("org.kde.quickcharts", 1);
    qmlRegisterAnonymousType<QQuickItem, 254>("org.kde.quickcharts", 1);
    qmlRegisterEnum<BarChart::WidthMode>("BarChart::WidthMode");
    qmlRegisterEnum<BarChart::Orientation>("BarChart::Orientation");
    qmlRegisterTypesAndRevisions<Chart>("org.kde.quickcharts", 1);
    qmlRegisterEnum<Chart::IndexingMode>("Chart::IndexingMode");
    qmlRegisterTypesAndRevisions<ChartAxisSource>("org.kde.quickcharts", 1);
    qmlRegisterEnum<ChartAxisSource::Axis>("ChartAxisSource::Axis");
    qmlRegisterTypesAndRevisions<ChartDataSource>("org.kde.quickcharts", 1);
    qmlRegisterTypesAndRevisions<ColorGradientSource>("org.kde.quickcharts", 1);
    qmlRegisterTypesAndRevisions<HistoryProxySource>("org.kde.quickcharts", 1);
    qmlRegisterEnum<HistoryProxySource::FillMode>("HistoryProxySource::FillMode");
    qmlRegisterTypesAndRevisions<LineChart>("org.kde.quickcharts", 1);
    qmlRegisterTypesAndRevisions<LineChartAttached>("org.kde.quickcharts", 1);
    qmlRegisterTypesAndRevisions<MapProxySource>("org.kde.quickcharts", 1);
    qmlRegisterTypesAndRevisions<ModelSource>("org.kde.quickcharts", 1);
    qmlRegisterTypesAndRevisions<PieChart>("org.kde.quickcharts", 1);
    qmlRegisterTypesAndRevisions<RangeGroup>("org.kde.quickcharts", 1);
    qmlRegisterTypesAndRevisions<SingleValueSource>("org.kde.quickcharts", 1);
    qmlRegisterTypesAndRevisions<XYChart>("org.kde.quickcharts", 1);
    qmlRegisterEnum<XYChart::Direction>("XYChart::Direction");
    QT_WARNING_POP
    qmlRegisterModule("org.kde.quickcharts", 1, 0);
}

static const QQmlModuleRegistration orgkdequickchartsRegistration("org.kde.quickcharts", qml_register_types_org_kde_quickcharts);
