/****************************************************************************
** Meta object code from reading C++ file 'HistoryProxySource.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kquickcharts-6.28.0/src/datasource/HistoryProxySource.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'HistoryProxySource.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18HistoryProxySourceE_t {};
} // unnamed namespace

template <> constexpr inline auto HistoryProxySource::qt_create_metaobjectdata<qt_meta_tag_ZN18HistoryProxySourceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "HistoryProxySource",
        "QML.Element",
        "auto",
        "sourceChanged",
        "",
        "itemChanged",
        "maximumHistoryChanged",
        "intervalChanged",
        "fillModeChanged",
        "clear",
        "source",
        "ChartDataSource*",
        "item",
        "maximumHistory",
        "interval",
        "fillMode",
        "FillMode",
        "DoNotFill",
        "FillFromStart",
        "FillFromEnd"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sourceChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'itemChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'maximumHistoryChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'intervalChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fillModeChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'source'
        QtMocHelpers::PropertyData<ChartDataSource*>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'item'
        QtMocHelpers::PropertyData<int>(12, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'maximumHistory'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'interval'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'fillMode'
        QtMocHelpers::PropertyData<enum FillMode>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'FillMode'
        QtMocHelpers::EnumData<enum FillMode>(16, 16, QMC::EnumFlags{}).add({
            {   17, FillMode::DoNotFill },
            {   18, FillMode::FillFromStart },
            {   19, FillMode::FillFromEnd },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<HistoryProxySource, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject HistoryProxySource::staticMetaObject = { {
    QMetaObject::SuperData::link<ChartDataSource::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18HistoryProxySourceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18HistoryProxySourceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18HistoryProxySourceE_t>.metaTypes,
    nullptr
} };

void HistoryProxySource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HistoryProxySource *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sourceChanged(); break;
        case 1: _t->itemChanged(); break;
        case 2: _t->maximumHistoryChanged(); break;
        case 3: _t->intervalChanged(); break;
        case 4: _t->fillModeChanged(); break;
        case 5: _t->clear(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (HistoryProxySource::*)()>(_a, &HistoryProxySource::sourceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (HistoryProxySource::*)()>(_a, &HistoryProxySource::itemChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (HistoryProxySource::*)()>(_a, &HistoryProxySource::maximumHistoryChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (HistoryProxySource::*)()>(_a, &HistoryProxySource::intervalChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (HistoryProxySource::*)()>(_a, &HistoryProxySource::fillModeChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ChartDataSource* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<ChartDataSource**>(_v) = _t->source(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->item(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->maximumHistory(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->interval(); break;
        case 4: *reinterpret_cast<enum FillMode*>(_v) = _t->fillMode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSource(*reinterpret_cast<ChartDataSource**>(_v)); break;
        case 1: _t->setItem(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setMaximumHistory(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setInterval(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setFillMode(*reinterpret_cast<enum FillMode*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *HistoryProxySource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HistoryProxySource::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18HistoryProxySourceE_t>.strings))
        return static_cast<void*>(this);
    return ChartDataSource::qt_metacast(_clname);
}

int HistoryProxySource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ChartDataSource::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void HistoryProxySource::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void HistoryProxySource::itemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void HistoryProxySource::maximumHistoryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void HistoryProxySource::intervalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void HistoryProxySource::fillModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
